@@ -76,9 +76,9 @@ authreadkeys(
 	keyid_t	keyno;
 	int	keytype;
 	char	buf[512];		/* lots of room for line */
-	u_char	keystr[20];
-	int	len;
-	int	j;
+	u_char	keystr[32];		/* Bug 2537 */
+	size_t	len;
+	size_t	j;
 
 	/*
 	 * Open file.  Complain and return if it can't be opened.
@@ -175,17 +175,17 @@ authreadkeys(
 			continue;
 		}
 		len = strlen(token);
-		if (len <= sizeof(keystr)) {
+		if (len <= 20) {	/* Bug 2537 */
 			MD5auth_setkey(keyno, keytype, (u_char *)token, len);
 		} else {
 			char	hex[] = "0123456789abcdef";
 			u_char	temp;
 			char	*ptr;
-			int	jlim;
+			size_t	jlim;
 
 			jlim = min(len, 2 * sizeof(keystr));
 			for (j = 0; j < jlim; j++) {
-				ptr = strchr(hex, tolower(token[j]));
+				ptr = strchr(hex, tolower((unsigned char)token[j]));
 				if (ptr == NULL)
 					break;	/* abort decoding */
 				temp = (u_char)(ptr - hex);
